//==================================================================================
// Projekt    : XPort Webserver
// Verfasser  : Thomas Berger
// Erstellt   : 15.05.2006
// Stand      : 28.05.2006, Version 1.0 tb
// Status     : Beta
//==================================================================================

// --- Includes ---
#include "global.h"


//-------------------------------------------------------------------------
//  ADC Initialisieren
//-------------------------------------------------------------------------
void ADC_Init(void){

  //TODO: Clock einstellen!

   ADCSRA = 0x00;
   //     | BV(ADEN)   //ADC aktivieren
   //     | BV(ADIE)   //Interrupt aktivieren
   //     | BV(ADPS2)  //ADC Clock Teiler
   //     | BV(ADPS1)  //ADC Clock Teiler
   //     | BV(ADPS02) //ADC Clock Teiler

  

  //Externe AREF verwenden
   ADMUX = 0x00;
  // Interne 2.56V Referenz verwenden
  // ADMUX = BV(REFS1) | BV(REFS0); 

}


//-------------------------------------------------------------------------
//  Kanal des ADC-Mux setzen
//-------------------------------------------------------------------------
void ADC_SetChn(u8 Nbr){
  u8 Val = ADMUX & 0xF0; // MUX0 bis MUX3 wegmaskieren


  // Ist der gewnschte Kanal vorhanden?
  if (Nbr < 8)
  {
    Val |= (Nbr & 0x0F);
  }

  ADMUX = Val;

}



//-------------------------------------------------------------------------
//  ADC ein- oder ausschalten
//-------------------------------------------------------------------------
void ADC_Enable(boolean Enable){
  u8 Val = ADCSRA;

  Val &= ~BV(ADEN); // Enable wegmaskieren
  Val &= ~BV(ADIE); // Interrupt wegmaskieren 

  if(Enable){
    Val |= BV(ADEN);
  }

  ADCSRA = Val;
}





//-------------------------------------------------------------------------
//  Einen Messwert lesen
//-------------------------------------------------------------------------
u16 ADC_GetSample(void){
  // Eine Wandlung "single conversion"
  ADCSRA |= BV(ADSC);
  // Messung beendet wenn Bit wieder gelscht
  while ( ADCSRA & BV(ADSC) );

  // Messwert zurck geben
  return ( ADCL+(ADCH<<8) );
}



//-------------------------------------------------------------------------
//  Einen Messwert mit Mittelwertbildung lesen
//-------------------------------------------------------------------------
u16 ADC_GetMeanChn(u8 Chn, u16 NbrSamp){
  u16 i;
  u32 Val=0;

  // Kanal whlen
  ADC_SetChn(Chn);

  //ADC Einschalten
  ADC_Enable(TRUE);

  //Die ersten paar Messungen verwerfen
  for(i=0;i<2;i++)
  {
    Val += (u32)ADC_GetSample;
  }


  Val = 0;

  //Nun den arithmetischen Mittelwert aus den Messungen bilden
  for(i=0;i<NbrSamp;i++)
  {
    Val += ADC_GetSample();
  }
  Val /= NbrSamp;


  //ADC ausschalten
  ADC_Enable(FALSE);


  return (u16)Val;

}





//-------------------------------------------------------------------------
//  Einen Messwert lesen
//-------------------------------------------------------------------------
u16 ADC_GetChn(u8 Chn){
  return ADC_GetMeanChn(Chn, 1);
}


