//==================================================================================
// Projekt    : XPort Webserver
// Verfasser  : Thomas Berger
// Erstellt   : 15.05.2006
// Stand      : 28.05.2006, Version 1.0 tb
// Status     : Beta
//==================================================================================


// --- Includes ---
#include <avr/wdt.h>
#include "global.h"
#include "mcu.h"


//-------------------------------------------------------------------------
// Den MCU initialisieren: Ports, Timer, Watchdog einstellen
//-------------------------------------------------------------------------
void mcu_init(void){

  // Port B
  DDRB  = 0x00;
  PORTB = 0xFF;

  // Port C
  DDRC  = 0x00;
  PORTC = 0xFF;

  // Port D
  DDRD  = BV(PD1); // TXD
  PORTD = 0xFF;

  // SPI
  /*
  SPCR = BV(SPIE)  // SPI Interrupt Enable
        |BV(SPE)   // SPI Enable
        |BV(CPOL);  // SCK High wenn nicht zu tun
    //  |BV(MSTR)  // Master Device
  */

  // UART Inizialisieren
  UCSRA=0x00;
  UCSRB=BV(RXCIE)|BV(TXCIE)|BV(RXEN)|BV(TXEN);
  UCSRC = BV(URSEL)  // UCSRC selektiert
       //|BV(UPM1)   // Gerade Paritt
         |BV(USBS)   // Zwei Stoppbit
         |BV(UCSZ1)
         |BV(UCSZ0); // Acht Bit

  // 25 = 9600 Baud bei  4 Mhz
  // 51 = 9600 Baud bei  8 Mhz
  // 64 = 9600 Baud bei 10 Mhz
  UBRRL = 64;
  UBRRH = 0;

  // Timer Inizialisieren
  //  bei CK/8
  //  32h ->   50d -> 100us  4MHz
  //  64h ->  100d -> 100us  8MHz
  //  7Dh ->  125d -> 100us 10MHz

  //  bei CK/64
  //  3Eh ->   62d ->   1ms  4MHz
  //  9Ch ->  156d ->   1ms 10MHz
  TCCR1B=BV(WGM12)|BV(CS11); //Timer reset bei Compare,CK/8;
  //TCCR1B=BV(WGM12)|BV(CS11)|BV(CS10); //Timer reset bei Compare,CK/64;
  OCR1AL=0x7d;
  TCNT1L=0x0000;            //Start/Reload Wert;
  TIMSK |= BV( OCIE1A);              //Interrupt Compare 1A

  // WatchDog Inizialisieren
  /*
  wdt_reset();
  // WatchDog, Reset nach ca. 1.9 Sekunden
  wdt_enable(BV(WDP0)|BV(WDP1)|BV(WDP2));
  */

  // Interrupts maskieren
  GICR=BV(INT1)|BV(INT0);     // INT0 und INT1 freigeben
  MCUCR=BV(ISC11)|BV(ISC01);  // Interrupt auf jeden Flanken wechsel

  //Interrupts freigeben
  sei ();
}


//-------------------------------------------------------------------------
// Watchdog zurcksetzen
//-------------------------------------------------------------------------
inline void mcu_watchdogreset(void){
  wdt_reset();
}

